package ru.itmo.stat.gibdd.aggregator.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import ru.itmo.stat.gibdd.aggregator.domain.dto.AccidentCardDto;
import ru.itmo.stat.gibdd.aggregator.parser.StatGibddParser;
import ru.itmo.stat.gibdd.aggregator.service.AccidentCardService;

@RestController
@RequiredArgsConstructor
@RequestMapping("/parse")
@Api(value = "ParsingController", description = "Operations with parsing")
public class ParsingController {
    private final StatGibddParser statGibddParser;
    private final AccidentCardService accidentCardService;

    @ApiOperation(value = "Parse data by entered parameters")
    @PostMapping(value = "/parseByParams")
    public void parseByParams(@PathVariable String reg, @PathVariable String parReg, @PathVariable String date) {
        statGibddParser.parseByParams(date, parReg, reg);
    }

    @PostMapping(value = "/parseSpb")
    public void parseSpb() {
        //String[] months = {"MONTHS:1.2021", "MONTHS:2.2021", "MONTHS:3.2021"};
        String[] months = {"MONTHS:1.2019", "MONTHS:2.2019", "MONTHS:3.2019", "MONTHS:4.2019",
                "MONTHS:5.2019", "MONTHS:6.2019", "MONTHS:7.2019", "MONTHS:8.2019",
                "MONTHS:9.2019", "MONTHS:10.2019", "MONTHS:11.2019", "MONTHS:12.2019",
                "MONTHS:1.2020", "MONTHS:2.2020", "MONTHS:3.2020", "MONTHS:4.2020",
                "MONTHS:5.2020", "MONTHS:6.2020", "MONTHS:7.2020", "MONTHS:8.2020",
                "MONTHS:9.2020", "MONTHS:10.2020", "MONTHS:11.2020", "MONTHS:12.2020"};
        String[] spbRegs = {"40262", "40263", "40265", "40270",
                "40273", "40276", "40277", "40278", "40279",
                "40280", "40281", "40284", "40285", "40288",
                "40290", "40294", "40296", "40298"};
        String[] loRegs = {"41203", "41206", "41209", "41212",
                "41215", "41218", "41221", "41224", "41225",
                "41227", "41230", "41233", "41236", "41239",
                "41242", "41245", "41248", "41254"};
        for (String m :
                months) {
            System.out.println("Started parsing for month:" + m);
            for (String r :
                    spbRegs) {
                statGibddParser.parseByParams(m, "40", r);
            }
            for (String r1 :
                    loRegs) {
                statGibddParser.parseByParams(m, "41", r1);
            }
        }
    }

    @PostMapping(value = "/testParsing")
    public void testParsing() {
        statGibddParser.parseByParams("MONTHS:2.2021", "4", "4204");
    }
}
