package ru.itmo.stat.gibdd.aggregator.parser;

import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;

import lombok.RequiredArgsConstructor;
import org.geolatte.geom.G2D;
import org.geolatte.geom.Point;
import ru.itmo.stat.gibdd.aggregator.domain.dto.AccidentCardDto;
import ru.itmo.stat.gibdd.aggregator.domain.dto.ParticipantDto;
import ru.itmo.stat.gibdd.aggregator.domain.dto.VehicleDto;
import ru.itmo.stat.gibdd.aggregator.service.*;

import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.time.LocalDate;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;

import static org.geolatte.geom.builder.DSL.g;
import static org.geolatte.geom.builder.DSL.point;
import static org.geolatte.geom.crs.CoordinateReferenceSystems.WGS84;

@RequiredArgsConstructor
public class StatGibddParser {
    private final AccidentTypeService accidentTypeService;
    private final RoadConditionService roadConditionService;
    private final IlluminationService illuminationService;
    private final DistrictService districtService;
    private final RoadTrafficChangeService roadTrafficChangeService;
    private final RoadImportanceService roadImportanceService;
    private final StreetCategoryService streetCategoryService;
    private final RoadService roadService;
    private final StreetService streetService;
    private final SettlementNameService settlementNameService;
    private final AccidentWeatherService accidentWeatherService;
    private final AccidentCardService accidentCardService;
    private final AccidentDrivingModeFactorService accidentDrivingModeFactorService;
    private final AccidentRoadMaintenanceDisadvantageService accidentRoadMaintenanceDisadvantageService;
    private final AccidentRoadNetworkObjectService accidentRoadNetworkObjectService;
    private final AccidentPlaceStatusService accidentPlaceStatusService;
    private final VehicleColorService vehicleColorService;
    private final VehicleDamageService vehicleDamageService;
    private final VehicleMalfunctionService vehicleMalfunctionService;
    private final VehicleModelService vehicleModelService;
    private final VehicleOwnershipService vehicleOwnershipService;
    private final VehicleService vehicleService;
    private final AccidentVehicleService accidentVehicleService;
    private final AccidentParticipantService accidentParticipantService;
    private final ParticipantCategoryService participantCategoryService;
    private final ParticipantMainTrafficViolationService participantMainTrafficViolationService;
    private final ParticipantRelatedTrafficViolationService participantRelatedTrafficViolationService;
    private final ParticipantService participantService;
    private final ParticipantStatusService participantStatusService;
    private final SeatTypeService seatTypeService;
    private final TrafficViolationService trafficViolationService;
    private final VehicleTypeService vehicleTypeService;

    public void parseByParams(String date, String parReg, String reg) {
        List<AccidentCardDto> accidentCardDtos = new ArrayList<>();
        String data = "{\"data\":\"{\\\"date\\\":[\\\"" + date + "\\\"],\\\"ParReg\\\":\\\"" + parReg + "\\\",\\\"order\\\":{\\\"type\\\":\\\"1\\\",\\\"fieldName\\\":\\\"dat\\\"},\\\"reg\\\":\\\"" + reg + "\\\",\\\"ind\\\":\\\"1\\\",\\\"st\\\":\\\"1\\\",\\\"en\\\":\\\"16\\\"}\"}";
        var client = HttpClient.newHttpClient();
        var request = HttpRequest.newBuilder(URI.create("http://stat.gibdd.ru/map/getDTPCardData"))
                .header("Content-Type", "application/json")
                .POST(HttpRequest.BodyPublishers.ofString(data))
                .build();
        HttpResponse<String> response = null;
        try {
            response = client.send(request, HttpResponse.BodyHandlers.ofString());
        } catch (IOException | InterruptedException e) {
            e.printStackTrace();
        }
        String resp = response.body().replace("\\\"", "\"");
        resp = resp.substring(9, resp.length() - 2);

        try {
            JsonObject jsonObject;
            try {
                jsonObject = JsonParser.parseString(resp).getAsJsonObject();
            } catch (IllegalStateException E) {
                return;
            }
            for (JsonElement je :
                    jsonObject.getAsJsonArray("tab")) {
                JsonObject jo = je.getAsJsonObject();
                AccidentCardDto accidentCardDto = new AccidentCardDto();
                Long cardId = jo.get("KartId").getAsLong();
                accidentCardDto.setId(cardId);
                accidentCardDto.setDate(LocalDate.parse(jo.get("date").getAsString(), DateTimeFormatter.ofPattern("dd.MM.yyyy")));
                accidentCardDto.setTime(LocalTime.parse(jo.get("Time").getAsString()));
                accidentCardDto.setDistrict(districtService.getDistrict(jo.get("District").getAsString()));
                accidentCardDto.setDeadCount(jo.get("POG").getAsInt());
                accidentCardDto.setInjuredCount(jo.get("RAN").getAsInt());
                accidentCardDto.setVehicleCount(jo.get("K_TS").getAsInt());
                accidentCardDto.setParticipantCount(jo.get("K_UCH").getAsInt());
                JsonObject joInfo = jo.get("infoDtp").getAsJsonObject();
                double latitude = joInfo.get("COORD_L").getAsDouble();
                double longitude = joInfo.get("COORD_W").getAsDouble();
                Point<G2D> coords;
                coords = point(WGS84, g(latitude, longitude));
                accidentCardDto.setAccidentCoordinates(coords);
                accidentCardDto.setAccidentType(accidentTypeService.getAccidentType(jo.get("DTP_V").getAsString()));
                accidentCardDto.setRoadCondition(roadConditionService.getRoadCondition(joInfo.get("s_pch").getAsString()));
                accidentCardDto.setIllumination(illuminationService.getIllumination(joInfo.get("osv").getAsString()));
                List<String> weather = new ArrayList<>();
                for (JsonElement w :
                        joInfo.getAsJsonArray("s_pog")) {
                    weather.add(w.getAsString());
                }

                List<String> drivingModeFactors = new ArrayList<>();
                for (JsonElement w :
                        joInfo.getAsJsonArray("factor")) {
                    drivingModeFactors.add(w.getAsString());
                }

                List<String> roadMaintenanceDisadvantages = new ArrayList<>();
                for (JsonElement w :
                        joInfo.getAsJsonArray("ndu")) {
                    roadMaintenanceDisadvantages.add(w.getAsString());
                }

                List<String> roadNetworkObjects = new ArrayList<>();
                for (JsonElement w :
                        joInfo.getAsJsonArray("sdor")) {
                    roadNetworkObjects.add(w.getAsString());
                }

                accidentCardDto.setAccidentS(joInfo.get("s_dtp").getAsString());
                accidentCardDto.setRoadTrafficChange(roadTrafficChangeService.getRoadTrafficChange(joInfo.get("change_org_motion").getAsString()));
                accidentCardDto.setRoadImportance(roadImportanceService.getRoadImportance(joInfo.get("dor_z").getAsString()));

                try {
                    accidentCardDto.setRoadCategory(joInfo.get("dor_k").getAsInt());
                } catch (Exception ignored) {
                }

                accidentCardDto.setStreetCategory(streetCategoryService.getStreetCategory(joInfo.get("k_ul").toString()));

                try {
                    accidentCardDto.setDistance(joInfo.get("km").getAsDouble() + (joInfo.get("m").getAsDouble() / 1000));
                } catch (Exception ignored) {
                }

                accidentCardDto.setRoad(roadService.getRoad(joInfo.get("dor").getAsString()));
                accidentCardDto.setHouse(joInfo.get("house").getAsString());
                accidentCardDto.setStreet(streetService.getStreet(joInfo.get("street").getAsString()));
                accidentCardDto.setSettlementName(settlementNameService.getSettlementName(joInfo.get("n_p").getAsString()));

                accidentCardService.addCard(accidentCardDto);
                accidentWeatherService.addAccidentWeather(cardId, weather);
                accidentDrivingModeFactorService.addAccidentDrivingModeFactor(cardId, drivingModeFactors);
                accidentRoadMaintenanceDisadvantageService.addAccidentRoadMaintenanceDisadvantage(cardId, roadMaintenanceDisadvantages);
                accidentRoadNetworkObjectService.addAccidentRoadNetworkObject(cardId, roadNetworkObjects);

                try {
                    for (JsonElement vje :
                            joInfo.getAsJsonArray("ts_info")) {
                        JsonObject vjo = vje.getAsJsonObject();
                        VehicleDto vehicleDto = new VehicleDto();
                        vehicleDto.setVehicleNumber(vjo.get("n_ts").getAsLong());
                        try {
                            vehicleDto.setManufactureYear(vjo.get("g_v").getAsInt());

                        } catch (Exception ignored) {
                        }
                        vehicleDto.setVehicleAccidentPlaceStatus(accidentPlaceStatusService.getAccidentPlaceStatusId(vjo.get("ts_s").getAsString()));
                        vehicleDto.setVehicleColor(vehicleColorService.getVehicleColorId(vjo.get("color").getAsString()));
                        vehicleDto.setVehicleDamage(vehicleDamageService.getVehicleDamageId(vjo.get("m_pov").getAsString()));
                        vehicleDto.setVehicleMalfunction(vehicleMalfunctionService.getVehicleMalfunctionId(vjo.get("t_n").getAsString()));
                        vehicleDto.setVehicleModel(vehicleModelService.getVehicleModelId(vjo.get("marka_ts").getAsString(),
                                vjo.get("m_ts").getAsString(), vjo.get("r_rul").getAsString()));
                        vehicleDto.setVehicleOwnership(vehicleOwnershipService.getVehicleOwnershipId(vjo.get("f_sob").getAsString(),
                                vjo.get("o_pf").getAsString()));
                        vehicleDto.setVehicleType(vehicleTypeService.getVehicleTypeId(vjo.get("t_ts").getAsString()));
                        String vehicleId = vehicleService.addVehicle(vehicleDto, cardId);
                        accidentVehicleService.addAccidentVehicle(cardId, vehicleId);

                        try {
                            for (JsonElement pje :
                                    vjo.getAsJsonArray("ts_uch")) {
                                JsonObject pjo = pje.getAsJsonObject();
                                ParticipantDto participantDto = new ParticipantDto();
                                participantDto.setParticipantCategory(participantCategoryService.getParticipantCategory(pjo.get("K_UCH").getAsString()));
                                participantDto.setParticipantStatus(participantStatusService.getParticipantStatus(pjo.get("S_T").getAsString()));
                                String sex = pjo.get("POL").getAsString();
                                if (sex.equals("Мужской")) {
                                    participantDto.setSex("MALE");
                                } else if (sex.equals("Женский")) {
                                    participantDto.setSex("FEMALE");
                                } else {
                                    participantDto.setSex(sex);
                                }
                                try {
                                    participantDto.setDrivingExperience(pjo.get("V_ST").getAsInt());
                                } catch (Exception ignored) {
                                }
                                try {
                                    participantDto.setAlcohol(pjo.get("ALCO").getAsInt());
                                } catch (Exception ignored) {
                                }
                                String belt = pjo.get("SAFETY_BELT").getAsString();
                                if (belt.equals("Да")) {
                                    participantDto.setSafetyBeltUsed(true);
                                } else if (belt.equals("Нет")) {
                                    participantDto.setSafetyBeltUsed(false);
                                }
                                participantDto.setAccidentPlaceStatus(accidentPlaceStatusService.getAccidentPlaceStatusId(pjo.get("S_SM").getAsString()));
                                participantDto.setParticipantNumber(pjo.get("N_UCH").getAsInt());
                                participantDto.setSeatType(seatTypeService.getSeatType(pjo.get("S_SEAT_GROUP").getAsString()));
                                participantDto.setInjuredCardId(pjo.get("INJURED_CARD_ID").getAsString());

                                String pId = participantService.addParticipant(participantDto, cardId);
                                accidentParticipantService.addAccidentParticipant(cardId, pId);

                                for (JsonElement w :
                                        pjo.getAsJsonArray("NPDD")) {
                                    participantMainTrafficViolationService.addParticipantMainTrafficViolation(pId, trafficViolationService.getTrafficViolation(w.getAsString()).getId());
                                }

                                for (JsonElement w :
                                        pjo.getAsJsonArray("SOP_NPDD")) {
                                    participantRelatedTrafficViolationService.addParticipantRelatedTrafficViolation(pId, trafficViolationService.getTrafficViolation(w.getAsString()).getId());
                                }


                            }
                        } catch (Exception e) {
                            e.printStackTrace();
                        }

                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }

                try {
                    for (JsonElement pje1 :
                            joInfo.getAsJsonArray("uchInfo")) {
                        JsonObject pjo1 = pje1.getAsJsonObject();
                        ParticipantDto participantDto = new ParticipantDto();
                        participantDto.setParticipantCategory(participantCategoryService.getParticipantCategory(pjo1.get("K_UCH").getAsString()));
                        participantDto.setParticipantStatus(participantStatusService.getParticipantStatus(pjo1.get("S_T").getAsString()));
                        String sex = pjo1.get("POL").getAsString();
                        if (sex.equals("Мужской")) {
                            participantDto.setSex("MALE");
                        } else if (sex.equals("Женский")) {
                            participantDto.setSex("FEMALE");
                        } else {
                            participantDto.setSex(sex);
                        }

                        try {
                            participantDto.setAlcohol(pjo1.get("ALCO").getAsInt());
                        } catch (Exception ignored) {
                        }
                        participantDto.setAccidentPlaceStatus(accidentPlaceStatusService.getAccidentPlaceStatusId(pjo1.get("S_SM").getAsString()));
                        participantDto.setParticipantNumber(pjo1.get("N_UCH").getAsInt());

                        String pId1 = participantService.addParticipant(participantDto, cardId);
                        accidentParticipantService.addAccidentParticipant(cardId, pId1);

                        for (JsonElement w :
                                pjo1.getAsJsonArray("NPDD")) {
                            participantMainTrafficViolationService.addParticipantMainTrafficViolation(pId1, trafficViolationService.getTrafficViolation(w.getAsString()).getId());
                        }

                        for (JsonElement w :
                                pjo1.getAsJsonArray("SOP_NPDD")) {
                            participantRelatedTrafficViolationService.addParticipantRelatedTrafficViolation(pId1, trafficViolationService.getTrafficViolation(w.getAsString()).getId());
                        }
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
