package ru.itmo.stat.gibdd.aggregator.service;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ru.itmo.stat.gibdd.aggregator.domain.dto.AccidentCardDto;
import ru.itmo.stat.gibdd.aggregator.domain.entities.AccidentCard;
import ru.itmo.stat.gibdd.aggregator.repository.AccidentCardRepository;

@Service
@Transactional
public class AccidentCardService extends AbstractService<String, AccidentCardDto, AccidentCard, AccidentCardRepository> {
    private final AccidentCardRepository accidentCardRepository;

    @Autowired
    public AccidentCardService(AccidentCardRepository repository, ModelMapper modelMapper, AccidentCardRepository accidentCardRepository) {
        super(repository, modelMapper);
        this.accidentCardRepository = accidentCardRepository;
    }

    @Override
    public Class<AccidentCard> getEntityClass() {
        return AccidentCard.class;
    }

    @Override
    public Class<AccidentCardDto> getDTOClass() {
        return AccidentCardDto.class;
    }

    public void addCard(AccidentCardDto accidentCardDto) {
        accidentCardRepository.save(modelMapper.map(accidentCardDto, getEntityClass()));
    }

    public AccidentCardDto getCard(Long id) {
        return modelMapper.map(accidentCardRepository.findAccidentCardById(id), getDTOClass());
    }

    public String getDeadCountInfo() {
        String response = "id;longitude;latitude;dead_count\n";
        for (AccidentCard accidentCard :
                accidentCardRepository.findAll()) {
            response += formLine(accidentCard.getId().toString(),
                    String.valueOf(accidentCard.getAccidentCoordinates().getPosition().getCoordinate(0)),
                    String.valueOf(accidentCard.getAccidentCoordinates().getPosition().getCoordinate(1)),
                    String.valueOf(accidentCard.getDeadCount()));
        }
        return response;
    }

    public String getCoordinatesInfo() {
        String response = "id;longitude;latitude\n";
        for (AccidentCard accidentCard :
                accidentCardRepository.findAll()) {
            response += formLineCoords(accidentCard.getId().toString(),
                    String.valueOf(accidentCard.getAccidentCoordinates().getPosition().getCoordinate(0)),
                    String.valueOf(accidentCard.getAccidentCoordinates().getPosition().getCoordinate(1)));
        }
        return response;
    }

    public String getVehicleCountInfo() {
        String response = "id;longitude;latitude;vehicle_count\n";
        for (AccidentCard accidentCard :
                accidentCardRepository.findAll()) {
            response += formLine(accidentCard.getId().toString(),
                    String.valueOf(accidentCard.getAccidentCoordinates().getPosition().getCoordinate(0)),
                    String.valueOf(accidentCard.getAccidentCoordinates().getPosition().getCoordinate(1)),
                    String.valueOf(accidentCard.getVehicleCount()));
        }
        return response;
    }

    public String getParticipantCountInfo() {
        String response = "id;longitude;latitude;participant_count\n";
        for (AccidentCard accidentCard :
                accidentCardRepository.findAll()) {
            response += formLine(accidentCard.getId().toString(),
                    String.valueOf(accidentCard.getAccidentCoordinates().getPosition().getCoordinate(0)),
                    String.valueOf(accidentCard.getAccidentCoordinates().getPosition().getCoordinate(1)),
                    String.valueOf(accidentCard.getParticipantCount()));
        }
        return response;
    }

    public String getAccidentTypeInfo() {
        String response = "id;longitude;latitude;accident_type\n";
        for (AccidentCard accidentCard :
                accidentCardRepository.findAll()) {
            response += formLine(accidentCard.getId().toString(),
                    String.valueOf(accidentCard.getAccidentCoordinates().getPosition().getCoordinate(0)),
                    String.valueOf(accidentCard.getAccidentCoordinates().getPosition().getCoordinate(1)),
                    String.valueOf(accidentCard.getAccidentType().getId()));
        }
        return response;
    }

    public String getRoadConditionsInfo() {
        String response = "id;longitude;latitude;road_conditions\n";
        for (AccidentCard accidentCard :
                accidentCardRepository.findAll()) {
            response += formLine(accidentCard.getId().toString(),
                    String.valueOf(accidentCard.getAccidentCoordinates().getPosition().getCoordinate(0)),
                    String.valueOf(accidentCard.getAccidentCoordinates().getPosition().getCoordinate(1)),
                    String.valueOf(accidentCard.getRoadCondition().getId()));
        }
        return response;
    }

    public String getIlluminationInfo() {
        String response = "id;longitude;latitude;illumination\n";
        for (AccidentCard accidentCard :
                accidentCardRepository.findAll()) {
            response += formLine(accidentCard.getId().toString(),
                    String.valueOf(accidentCard.getAccidentCoordinates().getPosition().getCoordinate(0)),
                    String.valueOf(accidentCard.getAccidentCoordinates().getPosition().getCoordinate(1)),
                    String.valueOf(accidentCard.getIllumination().getId()));
        }
        return response;
    }

    public String getCards() {
        String response = "id;longitude;latitude;date;time;district;dead_count;injured_count;vehicle_count;participant_count;accident_type;road_condition;illumination;road_traffic_changes;road_importance;road_category;street_category;road;house;street\n";
        for (AccidentCard accidentCard :
                accidentCardRepository.findAll()) {
            response = response + accidentCard.getId().toString() + ";" +
                    accidentCard.getAccidentCoordinates().getPosition().getCoordinate(0) + ";" +
                    accidentCard.getAccidentCoordinates().getPosition().getCoordinate(1) + ";" +
                    accidentCard.getDate() + ";" +
                    accidentCard.getTime() + ";" +
                    accidentCard.getDistrict().getDistrict() + ";" +
                    accidentCard.getDeadCount() + ";" +
                    accidentCard.getInjuredCount() + ";" +
                    accidentCard.getVehicleCount() + ";" +
                    accidentCard.getParticipantCount() + ";" +
                    accidentCard.getAccidentType().getAccidentType() + ";" +
                    accidentCard.getRoadCondition().getRoadCondition() + ";" +
                    accidentCard.getIllumination().getIllumination() + ";" +
                    accidentCard.getRoadTrafficChange().getRoadTrafficChange() + ";" +
                    accidentCard.getRoadImportance().getRoadImportance() + ";" +
                    accidentCard.getRoadCategory() + ";" +
                    accidentCard.getStreetCategory().getStreetCategory() + ";" +
                    accidentCard.getRoad().getRoad() + ";" +
                    accidentCard.getHouse() + ";" +
                    accidentCard.getStreet().getStreet() + "\n";
        }
        return response;
    }

    private static String formLine(String id, String lon, String lat, String info) {
        return id + ";" + lon + ";" + lat + ";" + info + "\n";
    }

    private static String formLineCoords(String id, String lon, String lat) {
        return id + ";" + lon + ";" + lat + "\n";
    }
}
