package ru.itmo.stat.gibdd.aggregator.service;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.itmo.stat.gibdd.aggregator.domain.dto.AccidentPlaceStatusDto;
import ru.itmo.stat.gibdd.aggregator.domain.entities.AccidentPlaceStatus;
import ru.itmo.stat.gibdd.aggregator.repository.AccidentPlaceStatusRepository;

import static ru.itmo.stat.gibdd.aggregator.utils.UUIDUtils.getUUIDFromString;

@Service
public class AccidentPlaceStatusService extends AbstractService<String, AccidentPlaceStatusDto, AccidentPlaceStatus, AccidentPlaceStatusRepository> {
    private final AccidentPlaceStatusRepository accidentPlaceStatusRepository;

    @Autowired
    public AccidentPlaceStatusService(AccidentPlaceStatusRepository repository, ModelMapper modelMapper, AccidentPlaceStatusRepository accidentPlaceStatusRepository) {
        super(repository, modelMapper);
        this.accidentPlaceStatusRepository = accidentPlaceStatusRepository;
    }

    @Override
    public Class<AccidentPlaceStatus> getEntityClass() {
        return AccidentPlaceStatus.class;
    }

    @Override
    public Class<AccidentPlaceStatusDto> getDTOClass() {
        return AccidentPlaceStatusDto.class;
    }

    public AccidentPlaceStatusDto getAccidentPlaceStatusId(String accidentPlaceStatus) {
        String uuid = getUUIDFromString(accidentPlaceStatus);
        if (accidentPlaceStatusRepository.existsById(uuid)) {
            return this.modelMapper.map(accidentPlaceStatusRepository.findAccidentPlaceStatusById(uuid), getDTOClass());
        } else {
            AccidentPlaceStatusDto accidentPlaceStatusDto = new AccidentPlaceStatusDto();
            accidentPlaceStatusDto.setStatus(accidentPlaceStatus);
            accidentPlaceStatusDto.setId(uuid);
            return this.create(accidentPlaceStatusDto);
        }
    }
}