package ru.itmo.stat.gibdd.aggregator.service;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.itmo.stat.gibdd.aggregator.domain.dto.AccidentTypeDto;
import ru.itmo.stat.gibdd.aggregator.domain.entities.AccidentType;
import ru.itmo.stat.gibdd.aggregator.repository.AccidentTypeRepository;

import static ru.itmo.stat.gibdd.aggregator.utils.UUIDUtils.getUUIDFromString;

@Service
public class AccidentTypeService extends AbstractService<String, AccidentTypeDto, AccidentType, AccidentTypeRepository> {
    private final AccidentTypeRepository accidentTypeRepository;

    @Autowired
    public AccidentTypeService(AccidentTypeRepository repository, ModelMapper modelMapper, AccidentTypeRepository accidentTypeRepository) {
        super(repository, modelMapper);
        this.accidentTypeRepository = accidentTypeRepository;
    }

    @Override
    public Class<AccidentType> getEntityClass() {
        return AccidentType.class;
    }

    @Override
    public Class<AccidentTypeDto> getDTOClass() {
        return AccidentTypeDto.class;
    }

    public AccidentTypeDto getAccidentType(String accidentType) {
        String uuid = getUUIDFromString(accidentType);
        if (accidentTypeRepository.existsById(uuid)) {
            return this.modelMapper.map(accidentTypeRepository.findAccidentTypeById(uuid), getDTOClass());
        } else {
            AccidentTypeDto accidentTypeDto = new AccidentTypeDto();
            accidentTypeDto.setAccidentType(accidentType);
            accidentTypeDto.setId(uuid);
            return this.create(accidentTypeDto);
        }
    }
}
