package ru.itmo.stat.gibdd.aggregator.service;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.itmo.stat.gibdd.aggregator.domain.dto.DistrictDto;
import ru.itmo.stat.gibdd.aggregator.domain.entities.District;
import ru.itmo.stat.gibdd.aggregator.repository.DistrictRepository;

import static ru.itmo.stat.gibdd.aggregator.utils.UUIDUtils.getUUIDFromString;

@Service
public class DistrictService extends AbstractService<String, DistrictDto, District, DistrictRepository> {
    private final DistrictRepository districtRepository;

    @Autowired
    public DistrictService(DistrictRepository repository, ModelMapper modelMapper, DistrictRepository districtRepository) {
        super(repository, modelMapper);
        this.districtRepository = districtRepository;
    }

    @Override
    public Class<District> getEntityClass() {
        return District.class;
    }

    @Override
    public Class<DistrictDto> getDTOClass() {
        return DistrictDto.class;
    }

    public DistrictDto getDistrict(String district) {
        String uuid = getUUIDFromString(district);
        if (districtRepository.existsById(uuid)) {
            return this.modelMapper.map(districtRepository.findDistrictById(uuid), getDTOClass());
        } else {
            DistrictDto districtDto = new DistrictDto();
            districtDto.setDistrict(district);
            districtDto.setId(uuid);
            return this.create(districtDto);
        }
    }
}