package ru.itmo.stat.gibdd.aggregator.service;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.itmo.stat.gibdd.aggregator.domain.dto.IlluminationDto;
import ru.itmo.stat.gibdd.aggregator.domain.entities.Illumination;
import ru.itmo.stat.gibdd.aggregator.repository.IlluminationRepository;


import static ru.itmo.stat.gibdd.aggregator.utils.UUIDUtils.getUUIDFromString;

@Service
public class IlluminationService extends AbstractService<String, IlluminationDto, Illumination, IlluminationRepository> {
    private final IlluminationRepository illuminationRepository;

    @Autowired
    public IlluminationService(IlluminationRepository repository, ModelMapper modelMapper, IlluminationRepository illuminationRepository) {
        super(repository, modelMapper);
        this.illuminationRepository = illuminationRepository;
    }

    @Override
    public Class<Illumination> getEntityClass() {
        return Illumination.class;
    }

    @Override
    public Class<IlluminationDto> getDTOClass() {
        return IlluminationDto.class;
    }

    public IlluminationDto getIllumination(String illumination) {
        String uuid = getUUIDFromString(illumination);
        if (illuminationRepository.existsById(uuid)) {
            return this.modelMapper.map(illuminationRepository.findIlluminationById(uuid), getDTOClass());
        } else {
            IlluminationDto illuminationDto = new IlluminationDto();
            illuminationDto.setIllumination(illumination);
            illuminationDto.setId(uuid);
            return this.create(illuminationDto);
        }
    }
}