package ru.itmo.stat.gibdd.aggregator.service;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.itmo.stat.gibdd.aggregator.domain.dto.ParticipantCategoryDto;
import ru.itmo.stat.gibdd.aggregator.domain.entities.ParticipantCategory;
import ru.itmo.stat.gibdd.aggregator.repository.ParticipantCategoryRepository;

import static ru.itmo.stat.gibdd.aggregator.utils.UUIDUtils.getUUIDFromString;

@Service
public class ParticipantCategoryService extends AbstractService<String, ParticipantCategoryDto, ParticipantCategory, ParticipantCategoryRepository> {
    private final ParticipantCategoryRepository participantCategoryRepository;

    @Autowired
    public ParticipantCategoryService(ParticipantCategoryRepository repository, ModelMapper modelMapper, ParticipantCategoryRepository participantCategoryRepository) {
        super(repository, modelMapper);
        this.participantCategoryRepository = participantCategoryRepository;
    }

    @Override
    public Class<ParticipantCategory> getEntityClass() {
        return ParticipantCategory.class;
    }

    @Override
    public Class<ParticipantCategoryDto> getDTOClass() {
        return ParticipantCategoryDto.class;
    }

    public ParticipantCategoryDto getParticipantCategory(String participantCategory) {
        String uuid = getUUIDFromString(participantCategory);
        if (participantCategoryRepository.existsById(uuid)) {
            return this.modelMapper.map(participantCategoryRepository.findParticipantCategoryById(uuid), getDTOClass());
        } else {
            ParticipantCategoryDto participantCategoryDto = new ParticipantCategoryDto();
            participantCategoryDto.setParticipantCategory(participantCategory);
            participantCategoryDto.setId(uuid);
            return this.create(participantCategoryDto);
        }
    }
}