package ru.itmo.stat.gibdd.aggregator.service;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.itmo.stat.gibdd.aggregator.domain.dto.ParticipantStatusDto;
import ru.itmo.stat.gibdd.aggregator.domain.entities.ParticipantStatus;
import ru.itmo.stat.gibdd.aggregator.repository.ParticipantStatusRepository;

import static ru.itmo.stat.gibdd.aggregator.utils.UUIDUtils.getUUIDFromString;

@Service
public class ParticipantStatusService extends AbstractService<String, ParticipantStatusDto, ParticipantStatus, ParticipantStatusRepository> {
    private final ParticipantStatusRepository participantStatusRepository;

    @Autowired
    public ParticipantStatusService(ParticipantStatusRepository repository, ModelMapper modelMapper, ParticipantStatusRepository participantStatusRepository) {
        super(repository, modelMapper);
        this.participantStatusRepository = participantStatusRepository;
    }

    @Override
    public Class<ParticipantStatus> getEntityClass() {
        return ParticipantStatus.class;
    }

    @Override
    public Class<ParticipantStatusDto> getDTOClass() {
        return ParticipantStatusDto.class;
    }

    public ParticipantStatusDto getParticipantStatus(String participantStatus) {
        String uuid = getUUIDFromString(participantStatus);
        if (participantStatusRepository.existsById(uuid)) {
            return this.modelMapper.map(participantStatusRepository.findParticipantStatusById(uuid), getDTOClass());
        } else {
            ParticipantStatusDto participantStatusDto = new ParticipantStatusDto();
            participantStatusDto.setParticipantStatus(participantStatus);
            participantStatusDto.setId(uuid);
            return this.create(participantStatusDto);
        }
    }
}