package ru.itmo.stat.gibdd.aggregator.service;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.itmo.stat.gibdd.aggregator.domain.dto.RoadConditionDto;
import ru.itmo.stat.gibdd.aggregator.domain.entities.RoadCondition;
import ru.itmo.stat.gibdd.aggregator.repository.RoadConditionRepository;

import static ru.itmo.stat.gibdd.aggregator.utils.UUIDUtils.getUUIDFromString;

@Service
public class RoadConditionService extends AbstractService<String, RoadConditionDto, RoadCondition, RoadConditionRepository> {
    private final RoadConditionRepository roadConditionRepository;

    @Autowired
    public RoadConditionService(RoadConditionRepository repository, ModelMapper modelMapper, RoadConditionRepository roadConditionRepository) {
        super(repository, modelMapper);
        this.roadConditionRepository = roadConditionRepository;
    }

    @Override
    public Class<RoadCondition> getEntityClass() {
        return RoadCondition.class;
    }

    @Override
    public Class<RoadConditionDto> getDTOClass() {
        return RoadConditionDto.class;
    }

    public RoadConditionDto getRoadCondition(String roadCondition) {
        String uuid = getUUIDFromString(roadCondition);
        if (roadConditionRepository.existsById(uuid)) {
            return this.modelMapper.map(roadConditionRepository.findRoadConditionById(uuid), getDTOClass());
        } else {
            RoadConditionDto roadConditionDto = new RoadConditionDto();
            roadConditionDto.setRoadCondition(roadCondition);
            roadConditionDto.setId(uuid);
            return this.create(roadConditionDto);
        }
    }
}
