package ru.itmo.stat.gibdd.aggregator.service;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.itmo.stat.gibdd.aggregator.domain.dto.RoadImportanceDto;
import ru.itmo.stat.gibdd.aggregator.domain.entities.RoadImportance;
import ru.itmo.stat.gibdd.aggregator.repository.RoadImportanceRepository;

import static ru.itmo.stat.gibdd.aggregator.utils.UUIDUtils.getUUIDFromString;

@Service
public class RoadImportanceService extends AbstractService<String, RoadImportanceDto, RoadImportance, RoadImportanceRepository> {
    private final RoadImportanceRepository roadImportanceRepository;

    @Autowired
    public RoadImportanceService(RoadImportanceRepository repository, ModelMapper modelMapper, RoadImportanceRepository roadImportanceRepository) {
        super(repository, modelMapper);
        this.roadImportanceRepository = roadImportanceRepository;
    }

    @Override
    public Class<RoadImportance> getEntityClass() {
        return RoadImportance.class;
    }

    @Override
    public Class<RoadImportanceDto> getDTOClass() {
        return RoadImportanceDto.class;
    }

    public RoadImportanceDto getRoadImportance(String roadImportance) {
        String uuid = getUUIDFromString(roadImportance);
        if (roadImportanceRepository.existsById(uuid)) {
            return this.modelMapper.map(roadImportanceRepository.findRoadImportanceById(uuid), getDTOClass());
        } else {
            RoadImportanceDto roadImportanceDto = new RoadImportanceDto();
            roadImportanceDto.setRoadImportance(roadImportance);
            roadImportanceDto.setId(uuid);
            return this.create(roadImportanceDto);
        }
    }
}