package ru.itmo.stat.gibdd.aggregator.service;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.itmo.stat.gibdd.aggregator.domain.dto.RoadMaintenanceDisadvantageDto;
import ru.itmo.stat.gibdd.aggregator.domain.entities.RoadMaintenanceDisadvantage;
import ru.itmo.stat.gibdd.aggregator.repository.RoadMaintenanceDisadvantageRepository;

import static ru.itmo.stat.gibdd.aggregator.utils.UUIDUtils.getUUIDFromString;

@Service
public class RoadMaintenanceDisadvantageService extends AbstractService<String, RoadMaintenanceDisadvantageDto, RoadMaintenanceDisadvantage, RoadMaintenanceDisadvantageRepository> {
    private final RoadMaintenanceDisadvantageRepository roadMaintenanceDisadvantageRepository;

    @Autowired
    public RoadMaintenanceDisadvantageService(RoadMaintenanceDisadvantageRepository repository, ModelMapper modelMapper, RoadMaintenanceDisadvantageRepository roadMaintenanceDisadvantageRepository) {
        super(repository, modelMapper);
        this.roadMaintenanceDisadvantageRepository = roadMaintenanceDisadvantageRepository;
    }

    @Override
    public Class<RoadMaintenanceDisadvantage> getEntityClass() {
        return RoadMaintenanceDisadvantage.class;
    }

    @Override
    public Class<RoadMaintenanceDisadvantageDto> getDTOClass() {
        return RoadMaintenanceDisadvantageDto.class;
    }

    public RoadMaintenanceDisadvantageDto getRoadMaintenanceDisadvantage(String roadMaintenanceDisadvantage) {
        String uuid = getUUIDFromString(roadMaintenanceDisadvantage);
        if (roadMaintenanceDisadvantageRepository.existsById(uuid)) {
            return this.modelMapper.map(roadMaintenanceDisadvantageRepository.findRoadMaintenanceDisadvantageById(uuid), getDTOClass());
        } else {
            RoadMaintenanceDisadvantageDto roadMaintenanceDisadvantageDto = new RoadMaintenanceDisadvantageDto();
            roadMaintenanceDisadvantageDto.setRoadMaintenanceDisadvantage(roadMaintenanceDisadvantage);
            roadMaintenanceDisadvantageDto.setId(uuid);
            return this.create(roadMaintenanceDisadvantageDto);
        }
    }
}