package ru.itmo.stat.gibdd.aggregator.service;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.itmo.stat.gibdd.aggregator.domain.dto.RoadNetworkObjectDto;
import ru.itmo.stat.gibdd.aggregator.domain.entities.RoadNetworkObject;
import ru.itmo.stat.gibdd.aggregator.repository.RoadNetworkObjectRepository;

import static ru.itmo.stat.gibdd.aggregator.utils.UUIDUtils.getUUIDFromString;

@Service
public class RoadNetworkObjectService extends AbstractService<String, RoadNetworkObjectDto, RoadNetworkObject, RoadNetworkObjectRepository> {
    private final RoadNetworkObjectRepository roadNetworkObjectRepository;

    @Autowired
    public RoadNetworkObjectService(RoadNetworkObjectRepository repository, ModelMapper modelMapper, RoadNetworkObjectRepository roadNetworkObjectRepository) {
        super(repository, modelMapper);
        this.roadNetworkObjectRepository = roadNetworkObjectRepository;
    }

    @Override
    public Class<RoadNetworkObject> getEntityClass() {
        return RoadNetworkObject.class;
    }

    @Override
    public Class<RoadNetworkObjectDto> getDTOClass() {
        return RoadNetworkObjectDto.class;
    }

    public RoadNetworkObjectDto getRoadNetworkObject(String roadNetworkObject) {
        String uuid = getUUIDFromString(roadNetworkObject);
        if (roadNetworkObjectRepository.existsById(uuid)) {
            return this.modelMapper.map(roadNetworkObjectRepository.findRoadNetworkObjectById(uuid), getDTOClass());
        } else {
            RoadNetworkObjectDto roadNetworkObjectDto = new RoadNetworkObjectDto();
            roadNetworkObjectDto.setRoadNetworkObject(roadNetworkObject);
            roadNetworkObjectDto.setId(uuid);
            return this.create(roadNetworkObjectDto);
        }
    }
}