package ru.itmo.stat.gibdd.aggregator.service;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.itmo.stat.gibdd.aggregator.domain.dto.RoadDto;
import ru.itmo.stat.gibdd.aggregator.domain.entities.Road;
import ru.itmo.stat.gibdd.aggregator.repository.RoadRepository;

import static ru.itmo.stat.gibdd.aggregator.utils.UUIDUtils.getUUIDFromString;

@Service
public class RoadService extends AbstractService<String, RoadDto, Road, RoadRepository> {
    private final RoadRepository roadRepository;

    @Autowired
    public RoadService(RoadRepository repository, ModelMapper modelMapper, RoadRepository roadRepository) {
        super(repository, modelMapper);
        this.roadRepository = roadRepository;
    }

    @Override
    public Class<Road> getEntityClass() {
        return Road.class;
    }

    @Override
    public Class<RoadDto> getDTOClass() {
        return RoadDto.class;
    }

    public RoadDto getRoad(String road) {
        String uuid = getUUIDFromString(road);
        if (roadRepository.existsById(uuid)) {
            return this.modelMapper.map(roadRepository.findRoadById(uuid), getDTOClass());
        } else {
            RoadDto roadDto = new RoadDto();
            roadDto.setRoad(road);
            roadDto.setId(uuid);
            return this.create(roadDto);
        }
    }
}