package ru.itmo.stat.gibdd.aggregator.service;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.itmo.stat.gibdd.aggregator.domain.dto.RoadTrafficChangeDto;
import ru.itmo.stat.gibdd.aggregator.domain.entities.RoadTrafficChange;
import ru.itmo.stat.gibdd.aggregator.repository.RoadTrafficChangeRepository;

import static ru.itmo.stat.gibdd.aggregator.utils.UUIDUtils.getUUIDFromString;

@Service
public class RoadTrafficChangeService extends AbstractService<String, RoadTrafficChangeDto, RoadTrafficChange, RoadTrafficChangeRepository> {
    private final RoadTrafficChangeRepository roadTrafficChangeRepository;

    @Autowired
    public RoadTrafficChangeService(RoadTrafficChangeRepository repository, ModelMapper modelMapper, RoadTrafficChangeRepository roadTrafficChangeRepository) {
        super(repository, modelMapper);
        this.roadTrafficChangeRepository = roadTrafficChangeRepository;
    }

    @Override
    public Class<RoadTrafficChange> getEntityClass() {
        return RoadTrafficChange.class;
    }

    @Override
    public Class<RoadTrafficChangeDto> getDTOClass() {
        return RoadTrafficChangeDto.class;
    }

    public RoadTrafficChangeDto getRoadTrafficChange(String roadTrafficChange) {
        String uuid = getUUIDFromString(roadTrafficChange);
        if (roadTrafficChangeRepository.existsById(uuid)) {
            return this.modelMapper.map(roadTrafficChangeRepository.findRoadTrafficChangeById(uuid), getDTOClass());
        } else {
            RoadTrafficChangeDto roadTrafficChangeDto = new RoadTrafficChangeDto();
            roadTrafficChangeDto.setRoadTrafficChange(roadTrafficChange);
            roadTrafficChangeDto.setId(uuid);
            return this.create(roadTrafficChangeDto);
        }
    }
}