package ru.itmo.stat.gibdd.aggregator.service;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.itmo.stat.gibdd.aggregator.domain.dto.SeatTypeDto;
import ru.itmo.stat.gibdd.aggregator.domain.entities.SeatType;
import ru.itmo.stat.gibdd.aggregator.repository.SeatTypeRepository;

import static ru.itmo.stat.gibdd.aggregator.utils.UUIDUtils.getUUIDFromString;

@Service
public class SeatTypeService extends AbstractService<String, SeatTypeDto, SeatType, SeatTypeRepository> {
    private final SeatTypeRepository seatTypeRepository;

    @Autowired
    public SeatTypeService(SeatTypeRepository repository, ModelMapper modelMapper, SeatTypeRepository seatTypeRepository) {
        super(repository, modelMapper);
        this.seatTypeRepository = seatTypeRepository;
    }

    @Override
    public Class<SeatType> getEntityClass() {
        return SeatType.class;
    }

    @Override
    public Class<SeatTypeDto> getDTOClass() {
        return SeatTypeDto.class;
    }

    public SeatTypeDto getSeatType(String seatType) {
        String uuid = getUUIDFromString(seatType);
        if (seatTypeRepository.existsById(uuid)) {
            return this.modelMapper.map(seatTypeRepository.findSeatTypeById(uuid), getDTOClass());
        } else {
            SeatTypeDto seatTypeDto = new SeatTypeDto();
            seatTypeDto.setSeatType(seatType);
            seatTypeDto.setId(uuid);
            return this.create(seatTypeDto);
        }
    }
}