package ru.itmo.stat.gibdd.aggregator.service;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.itmo.stat.gibdd.aggregator.domain.dto.SettlementNameDto;
import ru.itmo.stat.gibdd.aggregator.domain.entities.SettlementName;
import ru.itmo.stat.gibdd.aggregator.repository.SettlementNameRepository;

import static ru.itmo.stat.gibdd.aggregator.utils.UUIDUtils.getUUIDFromString;

@Service
public class SettlementNameService extends AbstractService<String, SettlementNameDto, SettlementName, SettlementNameRepository> {
    private final SettlementNameRepository settlementNameRepository;

    @Autowired
    public SettlementNameService(SettlementNameRepository repository, ModelMapper modelMapper, SettlementNameRepository settlementNameRepository) {
        super(repository, modelMapper);
        this.settlementNameRepository = settlementNameRepository;
    }

    @Override
    public Class<SettlementName> getEntityClass() {
        return SettlementName.class;
    }

    @Override
    public Class<SettlementNameDto> getDTOClass() {
        return SettlementNameDto.class;
    }

    public SettlementNameDto getSettlementName(String settlementName) {
        String uuid = getUUIDFromString(settlementName);
        if (settlementNameRepository.existsById(uuid)) {
            return this.modelMapper.map(settlementNameRepository.findSettlementNameById(uuid), getDTOClass());
        } else {
            SettlementNameDto settlementNameDto = new SettlementNameDto();
            settlementNameDto.setSettlementName(settlementName);
            settlementNameDto.setId(uuid);
            return this.create(settlementNameDto);
        }
    }
}