package ru.itmo.stat.gibdd.aggregator.service;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.itmo.stat.gibdd.aggregator.domain.dto.StreetCategoryDto;
import ru.itmo.stat.gibdd.aggregator.domain.entities.StreetCategory;
import ru.itmo.stat.gibdd.aggregator.repository.StreetCategoryRepository;

import static ru.itmo.stat.gibdd.aggregator.utils.UUIDUtils.getUUIDFromString;

@Service
public class StreetCategoryService extends AbstractService<String, StreetCategoryDto, StreetCategory, StreetCategoryRepository> {
    private final StreetCategoryRepository streetCategoryRepository;

    @Autowired
    public StreetCategoryService(StreetCategoryRepository repository, ModelMapper modelMapper, StreetCategoryRepository streetCategoryRepository) {
        super(repository, modelMapper);
        this.streetCategoryRepository = streetCategoryRepository;
    }

    @Override
    public Class<StreetCategory> getEntityClass() {
        return StreetCategory.class;
    }

    @Override
    public Class<StreetCategoryDto> getDTOClass() {
        return StreetCategoryDto.class;
    }

    public StreetCategoryDto getStreetCategory(String streetCategory) {
        String uuid = getUUIDFromString(streetCategory);
        if (streetCategoryRepository.existsById(uuid)) {
            return this.modelMapper.map(streetCategoryRepository.findStreetCategoryById(uuid), getDTOClass());
        } else {
            StreetCategoryDto streetCategoryDto = new StreetCategoryDto();
            streetCategoryDto.setStreetCategory(streetCategory);
            streetCategoryDto.setId(uuid);
            return this.create(streetCategoryDto);
        }
    }
}