package ru.itmo.stat.gibdd.aggregator.service;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.itmo.stat.gibdd.aggregator.domain.dto.StreetDto;
import ru.itmo.stat.gibdd.aggregator.domain.entities.Street;
import ru.itmo.stat.gibdd.aggregator.repository.StreetRepository;

import static ru.itmo.stat.gibdd.aggregator.utils.UUIDUtils.getUUIDFromString;

@Service
public class StreetService extends AbstractService<String, StreetDto, Street, StreetRepository> {
    private final StreetRepository streetRepository;

    @Autowired
    public StreetService(StreetRepository repository, ModelMapper modelMapper, StreetRepository streetRepository) {
        super(repository, modelMapper);
        this.streetRepository = streetRepository;
    }

    @Override
    public Class<Street> getEntityClass() {
        return Street.class;
    }

    @Override
    public Class<StreetDto> getDTOClass() {
        return StreetDto.class;
    }

    public StreetDto getStreet(String street) {
        String uuid = getUUIDFromString(street);
        if (streetRepository.existsById(uuid)) {
            return this.modelMapper.map(streetRepository.findStreetById(uuid), getDTOClass());
        } else {
            StreetDto streetDto = new StreetDto();
            streetDto.setStreet(street);
            streetDto.setId(uuid);
            return this.create(streetDto);
        }
    }
}
