package ru.itmo.stat.gibdd.aggregator.service;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.itmo.stat.gibdd.aggregator.domain.dto.TrafficViolationDto;
import ru.itmo.stat.gibdd.aggregator.domain.entities.TrafficViolation;
import ru.itmo.stat.gibdd.aggregator.repository.TrafficViolationRepository;

import static ru.itmo.stat.gibdd.aggregator.utils.UUIDUtils.getUUIDFromString;

@Service
public class TrafficViolationService extends AbstractService<String, TrafficViolationDto, TrafficViolation, TrafficViolationRepository> {
    private final TrafficViolationRepository trafficViolationRepository;

    @Autowired
    public TrafficViolationService(TrafficViolationRepository repository, ModelMapper modelMapper, TrafficViolationRepository trafficViolationRepository) {
        super(repository, modelMapper);
        this.trafficViolationRepository = trafficViolationRepository;
    }

    @Override
    public Class<TrafficViolation> getEntityClass() {
        return TrafficViolation.class;
    }

    @Override
    public Class<TrafficViolationDto> getDTOClass() {
        return TrafficViolationDto.class;
    }

    public TrafficViolationDto getTrafficViolation(String trafficViolation) {
        String uuid = getUUIDFromString(trafficViolation);
        if (trafficViolationRepository.existsById(uuid)) {
            return this.modelMapper.map(trafficViolationRepository.findTrafficViolationById(uuid), getDTOClass());
        } else {
            TrafficViolationDto trafficViolationDto = new TrafficViolationDto();
            trafficViolationDto.setViolation(trafficViolation);
            trafficViolationDto.setId(uuid);
            return this.create(trafficViolationDto);
        }
    }
}
