package ru.itmo.stat.gibdd.aggregator.service;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.itmo.stat.gibdd.aggregator.domain.dto.VehicleColorDto;
import ru.itmo.stat.gibdd.aggregator.domain.entities.VehicleColor;
import ru.itmo.stat.gibdd.aggregator.repository.VehicleColorRepository;

import static ru.itmo.stat.gibdd.aggregator.utils.UUIDUtils.getUUIDFromString;

@Service
public class VehicleColorService extends AbstractService<String, VehicleColorDto, VehicleColor, VehicleColorRepository> {
    private final VehicleColorRepository vehicleColorRepository;

    @Autowired
    public VehicleColorService(VehicleColorRepository repository, ModelMapper modelMapper, VehicleColorRepository vehicleColorRepository) {
        super(repository, modelMapper);
        this.vehicleColorRepository = vehicleColorRepository;
    }

    @Override
    public Class<VehicleColor> getEntityClass() {
        return VehicleColor.class;
    }

    @Override
    public Class<VehicleColorDto> getDTOClass() {
        return VehicleColorDto.class;
    }

    public VehicleColorDto getVehicleColorId(String vehicleColor) {
        String uuid = getUUIDFromString(vehicleColor);
        if (vehicleColorRepository.existsById(uuid)) {
            return this.modelMapper.map(vehicleColorRepository.findVehicleColorById(uuid), getDTOClass());
        } else {
            VehicleColorDto vehicleColorDto = new VehicleColorDto();
            vehicleColorDto.setColor(vehicleColor);
            vehicleColorDto.setId(uuid);
            return this.create(vehicleColorDto);
        }
    }
}
