package ru.itmo.stat.gibdd.aggregator.service;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.itmo.stat.gibdd.aggregator.domain.dto.VehicleDamageDto;
import ru.itmo.stat.gibdd.aggregator.domain.entities.VehicleDamage;
import ru.itmo.stat.gibdd.aggregator.repository.VehicleDamageRepository;

import static ru.itmo.stat.gibdd.aggregator.utils.UUIDUtils.getUUIDFromString;

@Service
public class VehicleDamageService extends AbstractService<String, VehicleDamageDto, VehicleDamage, VehicleDamageRepository> {
    private final VehicleDamageRepository vehicleDamageRepository;

    @Autowired
    public VehicleDamageService(VehicleDamageRepository repository, ModelMapper modelMapper, VehicleDamageRepository vehicleDamageRepository) {
        super(repository, modelMapper);
        this.vehicleDamageRepository = vehicleDamageRepository;
    }

    @Override
    public Class<VehicleDamage> getEntityClass() {
        return VehicleDamage.class;
    }

    @Override
    public Class<VehicleDamageDto> getDTOClass() {
        return VehicleDamageDto.class;
    }

    public VehicleDamageDto getVehicleDamageId(String vehicleDamage) {
        String uuid = getUUIDFromString(vehicleDamage);
        if (vehicleDamageRepository.existsById(uuid)) {
            return this.modelMapper.map(vehicleDamageRepository.findVehicleDamageById(uuid), getDTOClass());
        } else {
            VehicleDamageDto vehicleDamageDto = new VehicleDamageDto();
            vehicleDamageDto.setDamage(vehicleDamage);
            vehicleDamageDto.setId(uuid);
            return this.create(vehicleDamageDto);
        }
    }
}