package ru.itmo.stat.gibdd.aggregator.service;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.itmo.stat.gibdd.aggregator.domain.dto.VehicleMalfunctionDto;
import ru.itmo.stat.gibdd.aggregator.domain.entities.VehicleMalfunction;
import ru.itmo.stat.gibdd.aggregator.repository.VehicleMalfunctionRepository;

import static ru.itmo.stat.gibdd.aggregator.utils.UUIDUtils.getUUIDFromString;

@Service
public class VehicleMalfunctionService extends AbstractService<String, VehicleMalfunctionDto, VehicleMalfunction, VehicleMalfunctionRepository> {
    private final VehicleMalfunctionRepository vehicleMalfunctionRepository;

    @Autowired
    public VehicleMalfunctionService(VehicleMalfunctionRepository repository, ModelMapper modelMapper, VehicleMalfunctionRepository vehicleMalfunctionRepository) {
        super(repository, modelMapper);
        this.vehicleMalfunctionRepository = vehicleMalfunctionRepository;
    }

    @Override
    public Class<VehicleMalfunction> getEntityClass() {
        return VehicleMalfunction.class;
    }

    @Override
    public Class<VehicleMalfunctionDto> getDTOClass() {
        return VehicleMalfunctionDto.class;
    }

    public VehicleMalfunctionDto getVehicleMalfunctionId(String vehicleMalfunction) {
        String uuid = getUUIDFromString(vehicleMalfunction);
        if (vehicleMalfunctionRepository.existsById(uuid)) {
            return this.modelMapper.map(vehicleMalfunctionRepository.findVehicleMalfunctionById(uuid), getDTOClass());
        } else {
            VehicleMalfunctionDto vehicleMalfunctionDto = new VehicleMalfunctionDto();
            vehicleMalfunctionDto.setMalfunction(vehicleMalfunction);
            vehicleMalfunctionDto.setId(uuid);
            return this.create(vehicleMalfunctionDto);
        }
    }
}
