package ru.itmo.stat.gibdd.aggregator.service;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.itmo.stat.gibdd.aggregator.domain.dto.VehicleModelDto;
import ru.itmo.stat.gibdd.aggregator.domain.entities.VehicleModel;
import ru.itmo.stat.gibdd.aggregator.repository.VehicleModelRepository;

import static ru.itmo.stat.gibdd.aggregator.utils.UUIDUtils.getUUIDFromString;

@Service
public class VehicleModelService extends AbstractService<String, VehicleModelDto, VehicleModel, VehicleModelRepository> {
    private final VehicleModelRepository vehicleModelRepository;

    @Autowired
    public VehicleModelService(VehicleModelRepository repository, ModelMapper modelMapper, VehicleModelRepository vehicleModelRepository) {
        super(repository, modelMapper);
        this.vehicleModelRepository = vehicleModelRepository;
    }

    @Override
    public Class<VehicleModel> getEntityClass() {
        return VehicleModel.class;
    }

    @Override
    public Class<VehicleModelDto> getDTOClass() {
        return VehicleModelDto.class;
    }

    public VehicleModelDto getVehicleModelId(String vehicleBrand, String vehicleModel, String vehicleDriveType) {
        String uuid = getUUIDFromString(vehicleBrand + "-" + vehicleModel + "-" + vehicleDriveType);
        if (vehicleModelRepository.existsById(uuid)) {
            return this.modelMapper.map(vehicleModelRepository.findVehicleModelById(uuid), getDTOClass());
        } else {
            VehicleModelDto vehicleModelDto = new VehicleModelDto();
            vehicleModelDto.setBrand(vehicleBrand);
            vehicleModelDto.setModel(vehicleModel);
            vehicleModelDto.setDriveType(vehicleDriveType);
            vehicleModelDto.setId(uuid);
            return this.create(vehicleModelDto);
        }
    }
}
