package ru.itmo.stat.gibdd.aggregator.service;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.itmo.stat.gibdd.aggregator.domain.dto.VehicleOwnershipDto;
import ru.itmo.stat.gibdd.aggregator.domain.entities.VehicleOwnership;
import ru.itmo.stat.gibdd.aggregator.repository.VehicleOwnershipRepository;

import static ru.itmo.stat.gibdd.aggregator.utils.UUIDUtils.getUUIDFromString;

@Service
public class VehicleOwnershipService extends AbstractService<String, VehicleOwnershipDto, VehicleOwnership, VehicleOwnershipRepository> {
    private final VehicleOwnershipRepository vehicleOwnershipRepository;

    @Autowired
    public VehicleOwnershipService(VehicleOwnershipRepository repository, ModelMapper modelMapper, VehicleOwnershipRepository vehicleOwnershipRepository) {
        super(repository, modelMapper);
        this.vehicleOwnershipRepository = vehicleOwnershipRepository;
    }

    @Override
    public Class<VehicleOwnership> getEntityClass() {
        return VehicleOwnership.class;
    }

    @Override
    public Class<VehicleOwnershipDto> getDTOClass() {
        return VehicleOwnershipDto.class;
    }

    public VehicleOwnershipDto getVehicleOwnershipId(String vehicleOwnershipType, String vehicleOwnerType) {
        String uuid = getUUIDFromString(vehicleOwnershipType + "-" + vehicleOwnerType);
        if (vehicleOwnershipRepository.existsById(uuid)) {
            return this.modelMapper.map(vehicleOwnershipRepository.findVehicleOwnershipById(uuid), getDTOClass());
        } else {
            VehicleOwnershipDto vehicleOwnershipDto = new VehicleOwnershipDto();
            vehicleOwnershipDto.setOwnershipType(vehicleOwnershipType);
            vehicleOwnershipDto.setOwnerType(vehicleOwnerType);
            vehicleOwnershipDto.setId(uuid);
            return this.create(vehicleOwnershipDto);
        }
    }
}