package ru.itmo.stat.gibdd.aggregator.service;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.itmo.stat.gibdd.aggregator.domain.dto.VehicleTypeDto;
import ru.itmo.stat.gibdd.aggregator.domain.entities.VehicleType;
import ru.itmo.stat.gibdd.aggregator.repository.VehicleTypeRepository;

import static ru.itmo.stat.gibdd.aggregator.utils.UUIDUtils.getUUIDFromString;

@Service
public class VehicleTypeService extends AbstractService<String, VehicleTypeDto, VehicleType, VehicleTypeRepository> {
    private final VehicleTypeRepository vehicleTypeRepository;

    @Autowired
    public VehicleTypeService(VehicleTypeRepository repository, ModelMapper modelMapper, VehicleTypeRepository vehicleTypeRepository) {
        super(repository, modelMapper);
        this.vehicleTypeRepository = vehicleTypeRepository;
    }

    @Override
    public Class<VehicleType> getEntityClass() {
        return VehicleType.class;
    }

    @Override
    public Class<VehicleTypeDto> getDTOClass() {
        return VehicleTypeDto.class;
    }

    public VehicleTypeDto getVehicleTypeId(String vehicleType) {
        String uuid = getUUIDFromString(vehicleType);
        if (vehicleTypeRepository.existsById(uuid)) {
            return this.modelMapper.map(vehicleTypeRepository.findVehicleTypeById(uuid), getDTOClass());
        } else {
            VehicleTypeDto vehicleTypeDto = new VehicleTypeDto();
            vehicleTypeDto.setType(vehicleType);
            vehicleTypeDto.setId(uuid);
            return this.create(vehicleTypeDto);
        }
    }
}