package ru.itmo.stat.gibdd.aggregator.service;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.itmo.stat.gibdd.aggregator.domain.dto.WeatherDto;
import ru.itmo.stat.gibdd.aggregator.domain.entities.Weather;
import ru.itmo.stat.gibdd.aggregator.repository.WeatherRepository;

import static ru.itmo.stat.gibdd.aggregator.utils.UUIDUtils.getUUIDFromString;

@Service
public class WeatherService extends AbstractService<String, WeatherDto, Weather, WeatherRepository> {
    private final WeatherRepository weatherRepository;

    @Autowired
    public WeatherService(WeatherRepository repository, ModelMapper modelMapper, WeatherRepository weatherRepository) {
        super(repository, modelMapper);
        this.weatherRepository = weatherRepository;
    }

    @Override
    public Class<Weather> getEntityClass() {
        return Weather.class;
    }

    @Override
    public Class<WeatherDto> getDTOClass() {
        return WeatherDto.class;
    }

    public String getWeatherId(String weather) {
        String uuid = getUUIDFromString(weather);
        if (weatherRepository.existsById(uuid)) {
            return this.modelMapper.map(weatherRepository.findWeatherById(uuid), getDTOClass()).getId();
        } else {
            WeatherDto weatherDto = new WeatherDto();
            weatherDto.setWeather(weather);
            weatherDto.setId(uuid);
            return this.create(weatherDto).getId();
        }
    }
}